#include "common.h"
#include <tr1/unordered_map>
#include "tbb/spin_mutex.h"

using namespace std::tr1;

// Shared STL hash table that maps strings to ints
unordered_map<std::string, int> CoarseGrainStringTable;

// Global lock that protects the shared table.
tbb::spin_mutex global_mutex;

void CoarseGrainClear() {
    CoarseGrainStringTable.clear();
}

void CoarseGrainUpdate( const std::string& key ) {
    // Acquire lock on global_mutex
    tbb::spin_mutex::scoped_lock lock( global_mutex );
    // Update the map 
    CoarseGrainStringTable[key] += 1;
    // Destruction of lock object automatically releases the lock on global_mutex
}

int CoarseGrainGet( const std::string& key ) {
    unordered_map<std::string,int>::iterator k = CoarseGrainStringTable.find(key);
    return k!=CoarseGrainStringTable.end() ? k->second : 0;
}
