#include <cstdio>
using namespace std;

// Evaluate expression f(i) for each i in the half-open interval [lower,upper)
template<typename Functor>
void ForEach( int lower, int upper, const Functor& f ) {
    for( int i=lower; i<upper; ++i )
        f(i);
}

// Compute factorial of 4 
int LambdaFactorial() {
    int x=1;
    ForEach( 1, 
             5, 
             [&](int i) {
                 x*=i;
             });
    return x;
}

// Functor corresponding to result of the lambda expression in LambdaFactorial().
class MyFunctor {
    int& x;
public:

    // Constructor for the functor that corresponds to the lambda expression
    MyFunctor(int& x_) 
        : x(x_)
    {
    }

    // Member that defines calling the functor.
    void operator()(int i) const {
        x*=i;
    }
};
 
// Compute factorial of 4 
int ManualFactorial() {
    int x=1;
    ForEach( 1, 
             5, 
             MyFunctor(x) );
    return x;
}

int main() {
    printf("LambdaFactorial returns %d\n", LambdaFactorial());
    printf("ManualFactorial returns %d\n", ManualFactorial());
    return 0;
}
