#include "common.h"
#include <cmath>
#include <cstdio>
#include <cstdlib>
#include "tbb/tick_count.h"

static float A[N][N], B[N][N], C_Serial[N][N], C_Parallel[N][N];

static void initialize() {
    for (int i=0; i<N; ++i) {
        for (int j=0; j<N; ++j) {
            A[i][j] = (float)i+j;
            B[i][j] = (float)(i+1)*(j+1);
        }
    }
}

static void check() {
    for (int i=0; i<N; ++i) 
        for (int j=0; j<N; ++j) {
            float err = std::fabs(C_Parallel[i][j]-C_Serial[i][j]);
            if( err>=1E-6 ) {
                std::printf("Error: C_Parallel[%d][%d]=%g C_Serial=[%d][%d]=%g\n",
                            i, j, C_Parallel[i][j], i, j, C_Serial[i][j]);
                std::abort();
            }
        }
}

void time_one( const char* name, void (*routine)(float[N][N], const float[N][N], const float[N][N]), float c[N][N] ) {
    // Run once to warm up cache
    routine(c, A, B);
    // Run again to time it.
    tbb::tick_count before = tbb::tick_count::now ();
    routine(c, A, B);
    tbb::tick_count after = tbb::tick_count::now ();
    std::printf("%s:\t%g seconds\n", name, (after-before).seconds());
}

int main (void) {
    initialize();

    time_one( "SerialMatrixMultiply", SerialMatrixMultiply, C_Serial );
    time_one( "ParallelMatrixMultiply", ParallelMatrixMultiply, C_Parallel );

    check(); 
    return 0;
}
