#include <stddef.h>

const int MaxN = 1<<16;
const unsigned Radix = 10;
typedef int Digit;
const size_t CutOff = 16;

class TempSpace {
    static const size_t n = 1024;
    Digit temp[n];
    Digit* base;
public:
    operator Digit*() {return base;}
    TempSpace( size_t size ) {
        base = size<=n ? temp : new Digit[size];
    }
    ~TempSpace() {
        if( base!=temp ) 
            delete[] base;
    }
};

// Treating z, x, and y as polynomials of length n, compute z=x+y;
inline void Add( Digit z[], const Digit x[], const Digit y[], size_t n ) {
    for( size_t i=0; i<n; ++i ) 
        z[i] = x[i]+y[i];
    return;
}

// Treating z, x, and y as polynomials of length n, z-=x+y;
inline void AssignSubSub( Digit z[], const Digit x[], const Digit y[], size_t n ) {
    for( size_t i=0; i<n; ++i ) 
        z[i] -= x[i]+y[i];
}

void SimpleMul( Digit z[], const Digit x[], const Digit y[], size_t n );
void SerialKaratsubaMul( Digit z[], const Digit x[], const Digit y[], size_t n );
void ParallelKaratsubaMul( Digit z[], const Digit x[], const Digit y[], size_t n );
