#include "common.h"
#include "tbb/task_group.h"
using namespace tbb;

void ParallelKaratsubaMul( Digit z[], const Digit x[], const Digit y[], size_t n ) {
    if( n<=CutOff ) {
        SimpleMul( z, x, y, n );
    } else {
        size_t m = n/2;
        task_group g;
        if( m<=128 ) { 
            SerialKaratsubaMul( z, x, y, m );
            SerialKaratsubaMul( z+n, x+m, y+m, m );
        } else {
            g.run( [&]{ParallelKaratsubaMul( z, x, y, m );} );
            g.run( [&]{ParallelKaratsubaMul( z+n, x+m, y+m, m );} );
        } 
        TempSpace s(3*n);
        Digit* a = s+0;
        Digit* b = s+n;
        Digit* c = s+2*n;
        Add( a, x, x+m, m );
        Add( b, y, y+m, m ); 
        ParallelKaratsubaMul( c, a, b, m );
        g.wait();
        AssignSubSub( c, z, z+n, n );
        Add( z+m, z+m, c, n ); 
    }
}

